<?php

namespace Asadi\Subject\Http\Controllers\Api;

use App\Http\Controllers\Api\PublicController as BaseController;
use Asadi\Subject\Interfaces\SubjectRepositoryInterface;
use Asadi\Subject\Repositories\Presenter\SubjectItemTransformer;

/**
 * Pubic API controller class.
 */
class SubjectPublicController extends BaseController
{
    /**
     * Constructor.
     *
     * @param type \Asadi\Subject\Interfaces\SubjectRepositoryInterface $subject
     *
     * @return type
     */
    public function __construct(SubjectRepositoryInterface $subject)
    {
        $this->repository = $subject;
        parent::__construct();
    }

    /**
     * Show subject's list.
     *
     * @param string $slug
     *
     * @return response
     */
    protected function index()
    {
        $subjects = $this->repository
            ->setPresenter('\\Asadi\\Subject\\Repositories\\Presenter\\SubjectListPresenter')
            ->scopeQuery(function($query){
                return $query->orderBy('id','DESC');
            })->paginate();

        $subjects['code'] = 2000;
        return response()->json($subjects)
                ->setStatusCode(200, 'INDEX_SUCCESS');
    }

    /**
     * Show subject.
     *
     * @param string $slug
     *
     * @return response
     */
    protected function show($slug)
    {
        $subject = $this->repository
            ->scopeQuery(function($query) use ($slug) {
            return $query->orderBy('id','DESC')
                         ->where('slug', $slug);
        })->first(['*']);

        if (!is_null($subject)) {
            $subject         = $this->itemPresenter($module, new SubjectItemTransformer);
            $subject['code'] = 2001;
            return response()->json($subject)
                ->setStatusCode(200, 'SHOW_SUCCESS');
        } else {
            return response()->json([])
                ->setStatusCode(400, 'SHOW_ERROR');
        }

    }
}
