<?php

namespace Asadi\Subject\Http\Controllers;
use Litepie\Subject\Http\Requests\SubjectRequest;
use Litepie\Subject\Models\Subject;

trait SubjectWorkflow {
	
    /**
     * Workflow controller function for subject.
     *
     * @param Model   $subject
     * @param step    next step for the workflow.
     *
     * @return Response
     */

    public function putWorkflow(SubjectRequest $request, Subject $subject, $step)
    {

        try {

            $subject->updateWorkflow($step);

            return response()->json([
                'message'  => trans('messages.success.changed', ['Module' => trans('subject::subject.name'), 'status' => trans("app.{$step}")]),
                'code'     => 204,
                'redirect' => trans_url('/admin/subject/subject/' . $subject->getRouteKey()),
            ], 201);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/subject/subject/' . $subject->getRouteKey()),
            ], 400);

        }

    }

    /**
     * Workflow controller function for subject.
     *
     * @param Model   $subject
     * @param step    next step for the workflow.
     * @param user    encrypted user id.
     *
     * @return Response
    */

    public function getWorkflow(Subject $subject, $step, $user)
    {
        try {
            $user_id = decrypt($user);

            Auth::onceUsingId($user_id);

            $subject->updateWorkflow($step);

            $data = [
                'message' => trans('messages.success.changed', ['Module' => trans('subject::subject.name'), 'status' => trans("app.{$step}")]),
                'status'  => 'success',
                'step'    => trans("app.{$step}"),
            ];

            return $this->theme->layout('blank')->of('subject::admin.subject.message', $data)->render();

        } catch (ValidationException $e) {

            $data = [
                'message' => '<b>' . $e->getMessage() . '</b> <br /><br />' . implode('<br />', $e->validator->errors()->all()),
                'status'  => 'error',
                'step'    => trans("app.{$step}"),
            ];

            return $this->theme->layout('blank')->of('subject::admin.subject.message', $data)->render();

        } catch (Exception $e) {

            $data = [
                'message' => '<b>' . $e->getMessage(). '</b>',
                'status'  => 'error',
                'step'    => trans("app.{$step}"),
            ];

            return $this->theme->layout('blank')->of('subject::admin.subject.message', $data)->render();

        }

    }
}