<?php

namespace Asadi\Subject\Policies;

use App\User;
use Asadi\Subject\Models\Subject;

class SubjectPolicy
{

    /**
     * Determine if the given user can view the subject.
     *
     * @param User $user
     * @param Subject $subject
     *
     * @return bool
     */
    public function view(User $user, Subject $subject)
    {
        if ($user->canDo('subject.subject.view') && $user->is('admin')) {
            return true;
        }

        if ($user->canDo('blocks.block.view') 
        && $user->is('manager')
        && $block->user->parent_id == $user->id) {
            return true;
        }

        return $user->id === $subject->user_id;
    }

    /**
     * Determine if the given user can create a subject.
     *
     * @param User $user
     * @param Subject $subject
     *
     * @return bool
     */
    public function create(User $user)
    {
        return  $user->canDo('subject.subject.create');
    }

    /**
     * Determine if the given user can update the given subject.
     *
     * @param User $user
     * @param Subject $subject
     *
     * @return bool
     */
    public function update(User $user, Subject $subject)
    {
        if ($user->canDo('subject.subject.update') && $user->is('admin')) {
            return true;
        }

        if ($user->canDo('blocks.block.update') 
        && $user->is('manager')
        && $block->user->parent_id == $user->id) {
            return true;
        }

        return $user->id === $subject->user_id;
    }

    /**
     * Determine if the given user can delete the given subject.
     *
     * @param User $user
     * @param Subject $subject
     *
     * @return bool
     */
    public function destroy(User $user, Subject $subject)
    {
        if ($user->canDo('subject.subject.delete') && $user->is('admin')) {
            return true;
        }

        if ($user->canDo('blocks.block.delete') 
        && $user->is('manager')
        && $block->user->parent_id == $user->id) {
            return true;
        }

        return $user->id === $subject->user_id;
    }

    /**
     * Determine if the given user can verify the given subject.
     *
     * @param User $user
     * @param Subject $subject
     *
     * @return bool
     */
    public function verify(User $user, Subject $subject)
    {
        if ($user->canDo('subject.subject.verify') && $user->is('admin')) {
            return true;
        }

        if ($user->canDo('subject.subject.verify') 
        && $user->is('manager')
        && $subject->user->parent_id == $user->id) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given subject.
     *
     * @param User $user
     * @param Subject $subject
     *
     * @return bool
     */
    public function approve(User $user, Subject $subject)
    {
        if ($user->canDo('subject.subject.approve') && $user->is('admin')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperUser()) {
            return true;
        }
    }
}
