<?php

namespace Asadi\Subject\Workflow;

use Exception;
use Litepie\Workflow\Exceptions\WorkflowActionNotPerformedException;

use Asadi\Subject\Models\Subject;

class SubjectAction
{
    /**
     * Perform the complete action.
     *
     * @param Subject $subject
     *
     * @return Subject
     */
    public function complete(Subject $subject)
    {
        try {
            $subject->status = 'complete';
            return $subject->save();
        } catch (Exception $e) {
            throw new WorkflowActionNotPerformedException();
        }
    }

    /**
     * Perform the verify action.
     *
     * @param Subject $subject
     *
     * @return Subject
     */public function verify(Subject $subject)
    {
        try {
            $subject->status = 'verify';
            return $subject->save();
        } catch (Exception $e) {
            throw new WorkflowActionNotPerformedException();
        }
    }

    /**
     * Perform the approve action.
     *
     * @param Subject $subject
     *
     * @return Subject
     */public function approve(Subject $subject)
    {
        try {
            $subject->status = 'approve';
            return $subject->save();
        } catch (Exception $e) {
            throw new WorkflowActionNotPerformedException();
        }
    }

    /**
     * Perform the publish action.
     *
     * @param Subject $subject
     *
     * @return Subject
     */public function publish(Subject $subject)
    {
        try {
            $subject->status = 'publish';
            return $subject->save();
        } catch (Exception $e) {
            throw new WorkflowActionNotPerformedException();
        }
    }

    /**
     * Perform the archive action.
     *
     * @param Subject $subject
     *
     * @return Subject
     */
    public function archive(Subject $subject)
    {
        try {
            $subject->status = 'archive';
            return $subject->save();
        } catch (Exception $e) {
            throw new WorkflowActionNotPerformedException();
        }
    }

    /**
     * Perform the unpublish action.
     *
     * @param Subject $subject
     *
     * @return Subject
     */
    public function unpublish(Subject $subject)
    {
        try {
            $subject->status = 'unpublish';
            return $subject->save();
        } catch (Exception $e) {
            throw new WorkflowActionNotPerformedException();
        }
    }
}
