<?php

namespace Asadi\Subject\Workflow;

use Asadi\Subject\Models\Subject;
use Validator;

class SubjectValidator
{

    /**
     * Determine if the given subject is valid for complete status.
     *
     * @param Subject $subject
     *
     * @return bool / Validator
     */
    public function complete(Subject $subject)
    {
        return Validator::make($subject->toArray(), [
            'title' => 'required|min:15',
        ]);
    }

    /**
     * Determine if the given subject is valid for verify status.
     *
     * @param Subject $subject
     *
     * @return bool / Validator
     */
    public function verify(Subject $subject)
    {
        return Validator::make($subject->toArray(), [
            'title'  => 'required|min:15',
            'status' => 'in:complete',
        ]);
    }

    /**
     * Determine if the given subject is valid for approve status.
     *
     * @param Subject $subject
     *
     * @return bool / Validator
     */
    public function approve(Subject $subject)
    {
        return Validator::make($subject->toArray(), [
            'title'  => 'required|min:15',
            'status' => 'in:verify',
        ]);

    }

    /**
     * Determine if the given subject is valid for publish status.
     *
     * @param Subject $subject
     *
     * @return bool / Validator
     */
    public function publish(Subject $subject)
    {
        return Validator::make($subject->toArray(), [
            'title'       => 'required|min:15',
            'description' => 'required|min:50',
            'status'      => 'in:approve,archive,unpublish',
        ]);

    }

    /**
     * Determine if the given subject is valid for archive status.
     *
     * @param Subject $subject
     *
     * @return bool / Validator
     */
    public function archive(Subject $subject)
    {
        return Validator::make($subject->toArray(), [
            'title'       => 'required|min:15',
            'description' => 'required|min:50',
            'status'      => 'in:approve,publish,unpublish',
        ]);

    }

    /**
     * Determine if the given subject is valid for unpublish status.
     *
     * @param Subject $subject
     *
     * @return bool / Validator
     */
    public function unpublish(Subject $subject)
    {
        return Validator::make($subject->toArray(), [
            'title'       => 'required|min:15',
            'description' => 'required|min:50',
            'status'      => 'in:publish',
        ]);

    }
}
